/***************************************
	Auteur : Pierre Aubert
	Mail : pierre.aubert@lapp.in2p3.fr
	Licence : CeCILL-C
****************************************/
#include <iostream>
#include <vector>
#include <numeric>

// #include <ranges>
#include <execution>
#include <algorithm>

#include "micro_benchmark.h"

///Defines a vector of index
typedef std::vector<int> VecIndex;
///Defines a vector of data
typedef std::vector<float> VecData;

///Compute the calibration
/**	@param[out] vecCalibSignal : vector of calibrated signal
 * 	@param vecADC : vector of ADC values
 * 	@param tabPed : pointer to the table of pedestal
 * 	@param tabGain : pointer to the table of gain
 * 	@param vecIdx : vector of index to be used to round robin over the pixel
*/
void compute_calibration(VecData & vecCalibSignal,
			const VecData & vecADC, const float * tabPed, const float * tabGain, const VecIndex & vecIdx)
{
	std::transform(std::execution::par_unseq, std::begin(vecADC), std::end(vecADC), std::begin(vecIdx), std::begin(vecCalibSignal),
		[=](float adc, int i){
			return (adc - tabPed[i])*tabGain[i];
	});
}

///Get the number of nanoseconds per elements of the Calibration
/**	@param nbEvent : number of events of the tables
*/
void evaluateCalibration(size_t nbEvent){
	//Let's define size of data :
	size_t  nbPixel(NB_PIXEL), nbSlice(NB_SLICE);
	size_t nbElement(nbEvent*nbSlice*nbPixel);

	VecData vecGain(nbPixel), vecPedestal(nbPixel);
	std::fill(vecGain.begin(), vecGain.end(), 0.02f);
	std::fill(vecPedestal.begin(), vecPedestal.end(), 40.0f);
	
	VecData vecADCSignal(nbElement), vecCalibSignal(nbElement);
	std::fill(vecADCSignal.begin(), vecADCSignal.end(), 42.0f);
	
	VecIndex vecIdx(nbElement);		//Init vector of index for the computation
	std::iota(vecIdx.begin(), vecIdx.end(), 0);	//Hope some day views will work to avoid allocation of index vector
	std::transform(std::execution::par_unseq, vecIdx.begin(), vecIdx.end(), vecIdx.begin(),
		[=](int i){
			//Modulo computation, only once at the initialisation time
			return i % nbPixel;
		}
	);
	float *tabGain = vecGain.data(), *tabPed = vecPedestal.data();
	
	size_t fullNbElement(nbElement);
	micro_benchmarkAutoNsPrint("evaluateCalibration nvc++ pre computed modulo", fullNbElement, compute_calibration, vecCalibSignal, vecADCSignal, tabPed, tabGain, vecIdx);
}

int main(int argc, char** argv){
	return micro_benchmarkParseArg(argc, argv, evaluateCalibration);
}



