/***************************************
	Auteur : Pierre Aubert
	Mail : pierre.aubert@lapp.in2p3.fr
	Licence : CeCILL-C
****************************************/
#include <iostream>
#include <string_view>
#include <vector>
// #include <numeric>
#include <ranges>

#include <execution>
#include <algorithm>

#include "micro_benchmark.h"

///Compute the calibration
/**	@param[out] vecCalibSignal : vector of calibrated signal
 * 	@param vecADC : pointer to the table of ADC values
 * 	@param vecPed : pointer to the table of pedestal
 * 	@param vecGain : pointer to the table of gain
*/
void compute_calibration(std::vector<float> & vecCalibSignal,
			const std::vector<float> & vecADC, const std::vector<float> & vecPed, const std::vector<float> & vecGain)
{
// 	size_t nbElement = vecADC.size();
// 	size_t nbPixel = vecPed.size();
// 	size_t nbRepeat = nbElement / nbPixel;
	
// 	auto repeatPed = std::views::chunk(std::views::repeat(vecPed, nbRepeat), nbElement);
// 	auto repeatPed = std::views::join(std::views::repeat(vecPed, 20));
	auto repeatPed = std::views::join(std::views::repeat(vecPed));
// 	auto repeatPed = std::ranges::join_view(std::views::repeat(vecPed, nbRepeat));
// 	auto repeatPed = std::views::repeat(vecPed, nbRepeat) | std::views::join;
	
	std::transform(std::execution::par_unseq, std::begin(vecADC), std::end(vecADC), std::begin(repeatPed), vecCalibSignal.begin(),
		       [](float ped, float adc){
				return (adc - ped);
	});
	
	auto repeatGain = std::views::join(std::views::repeat(vecGain));
	std::transform(std::execution::par_unseq, std::begin(vecCalibSignal), std::end(vecCalibSignal), repeatGain.begin(), vecCalibSignal.begin(),
		       [](float baseline, float gain){
				return baseline*gain;
	});
}

///Get the number of nanoseconds per elements of the Calibration
/**	@param nbEvent : number of events of the tables
*/
void evaluateCalibration(size_t nbEvent){
	//Let's define size of data :
	size_t nbPixel(NB_PIXEL), nbSlice(NB_SLICE);
	size_t nbElement(nbEvent*nbSlice*nbPixel);

	std::vector<float> vecGain(nbPixel), vecPedestal(nbPixel);
	std::fill(vecGain.begin(), vecGain.end(), 0.02f);
	std::fill(vecPedestal.begin(), vecPedestal.end(), 40.0f);

	std::vector<float> vecADCSignal(nbElement), vecCalibSignal(nbElement);
	std::fill(vecADCSignal.begin(), vecADCSignal.end(), 42.0f);

	size_t fullNbElement(nbElement);
	micro_benchmarkAutoNsPrint("evaluateCalibration nvc++", fullNbElement, compute_calibration, vecCalibSignal, vecADCSignal, vecPedestal, vecGain);
}

int main(int argc, char** argv){
	return micro_benchmarkParseArg(argc, argv, evaluateCalibration);
}